package cn.com.infosec.jce.provider;

import cn.com.infosec.asn1.ASN1InputStream;
import cn.com.infosec.asn1.DERObject;
import cn.com.infosec.asn1.x509.CRLDistPoint;
import cn.com.infosec.asn1.x509.DistributionPoint;
import cn.com.infosec.asn1.x509.DistributionPointName;
import cn.com.infosec.asn1.x509.GeneralName;
import cn.com.infosec.asn1.x509.GeneralNames;
import cn.com.infosec.asn1.x509.ReasonFlags;
import cn.com.infosec.asn1.x509.TargetInformation;
import cn.com.infosec.asn1.x509.X509Extensions;
import cn.com.infosec.jce.exception.ExtCertPathValidatorException;
import cn.com.infosec.x509.ExtendedPKIXBuilderParameters;
import cn.com.infosec.x509.ExtendedPKIXParameters;
import cn.com.infosec.x509.PKIXAttrCertChecker;
import cn.com.infosec.x509.X509AttributeCertificate;
import cn.com.infosec.x509.X509CertStoreSelector;
import java.io.IOException;
import java.security.InvalidAlgorithmParameterException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.Principal;
import java.security.PublicKey;
import java.security.cert.CertPath;
import java.security.cert.CertPathBuilder;
import java.security.cert.CertPathBuilderException;
import java.security.cert.CertPathBuilderResult;
import java.security.cert.CertPathValidator;
import java.security.cert.CertPathValidatorException;
import java.security.cert.CertPathValidatorResult;
import java.security.cert.CertificateExpiredException;
import java.security.cert.CertificateNotYetValidException;
import java.security.cert.TrustAnchor;
import java.security.cert.X509CRL;
import java.security.cert.X509Certificate;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import javax.security.auth.x500.X500Principal;

class RFC3281CertPathUtilities {
   private static final String TARGET_INFORMATION;
   private static final String NO_REV_AVAIL;
   private static final String CRL_DISTRIBUTION_POINTS;
   private static final String AUTHORITY_INFO_ACCESS;

   static {
      TARGET_INFORMATION = X509Extensions.TargetInformation.getId();
      NO_REV_AVAIL = X509Extensions.NoRevAvail.getId();
      CRL_DISTRIBUTION_POINTS = X509Extensions.CRLDistributionPoints.getId();
      AUTHORITY_INFO_ACCESS = X509Extensions.AuthorityInfoAccess.getId();
   }

   protected static void processAttrCert7(X509AttributeCertificate attrCert, CertPath certPath, CertPath holderCertPath, ExtendedPKIXParameters pkixParams) {
      Set var4;
      String var5;
      if ((var4 = attrCert.getCriticalExtensionOIDs()).contains(var5 = TARGET_INFORMATION)) {
         try {
            TargetInformation.getInstance(CertPathValidatorUtilities.getExtensionValue(attrCert, var5));
         } catch (AnnotatedException var6) {
            throw new ExtCertPathValidatorException("Target information extension could not be read.", var6);
         } catch (IllegalArgumentException var7) {
            throw new ExtCertPathValidatorException("Target information extension could not be read.", var7);
         }
      }

      var4.remove(var5);
      Iterator pkixParams1 = pkixParams.getAttrCertCheckers().iterator();

      while(pkixParams1.hasNext()) {
         ((PKIXAttrCertChecker)pkixParams1.next()).check(attrCert, certPath, holderCertPath, var4);
      }

      if (!var4.isEmpty()) {
         throw new CertPathValidatorException("Attribute certificate contains unsupported critical extensions: " + var4);
      }
   }

   protected static void checkCRLs(X509AttributeCertificate attrCert, ExtendedPKIXParameters paramsPKIX, X509Certificate issuerCert, Date validDate, List certPathCerts) {
      if (paramsPKIX.isRevocationEnabled()) {
         if (attrCert.getExtensionValue(NO_REV_AVAIL) == null) {
            CRLDistPoint var10000;
            try {
               var10000 = CRLDistPoint.getInstance(CertPathValidatorUtilities.getExtensionValue(attrCert, CRL_DISTRIBUTION_POINTS));
            } catch (AnnotatedException var14) {
               throw new CertPathValidatorException("CRL distribution point extension could not be read.", var14);
            }

            CRLDistPoint var5;
            CRLDistPoint var10001 = var5 = var10000;

            try {
               CertPathValidatorUtilities.addAdditionalStoresFromCRLDistributionPoint(var10001, paramsPKIX);
            } catch (AnnotatedException var13) {
               throw new CertPathValidatorException("No additional CRL locations could be decoded from CRL distribution point extension.", var13);
            }

            CertStatus var6;
            var6 = new CertStatus.<init>();
            ReasonsMask var7;
            var7 = new ReasonsMask.<init>();
            AnnotatedException var8 = null;
            boolean var9 = false;
            ExtendedPKIXParameters var10002;
            AnnotatedException var33;
            boolean var35;
            if (var10000 != null) {
               DistributionPoint[] var32;
               try {
                  var32 = var5.getDistributionPoints();
               } catch (Exception var12) {
                  throw new ExtCertPathValidatorException("Distribution points could not be read.", var12);
               }

               DistributionPoint[] var28 = var32;
               int var10 = 0;

               while(true) {
                  label141: {
                     label158: {
                        int var34;
                        int var36;
                        try {
                           var34 = var10;
                           var36 = var28.length;
                        } catch (AnnotatedException var24) {
                           var33 = var24;
                           var35 = false;
                           break label158;
                        }

                        if (var34 >= var36) {
                           break;
                        }

                        try {
                           var34 = var6.getCertStatus();
                        } catch (AnnotatedException var23) {
                           var33 = var23;
                           var35 = false;
                           break label158;
                        }

                        if (var34 != 11) {
                           break;
                        }

                        boolean var37;
                        try {
                           var37 = var7.isAllReasons();
                        } catch (AnnotatedException var22) {
                           var33 = var22;
                           var35 = false;
                           break label158;
                        }

                        if (var37) {
                           break;
                        }

                        try {
                           var32 = var28;
                           var36 = var10;
                           var10002 = (ExtendedPKIXParameters)paramsPKIX.clone();
                        } catch (AnnotatedException var21) {
                           var33 = var21;
                           var35 = false;
                           break label158;
                        }

                        ExtendedPKIXParameters var11 = var10002;

                        try {
                           checkCRL(var32[var36], attrCert, var11, validDate, issuerCert, var6, var7, certPathCerts);
                           break label141;
                        } catch (AnnotatedException var20) {
                           var33 = var20;
                           var35 = false;
                        }
                     }

                     AnnotatedException var29 = var33;
                     var8 = new AnnotatedException.<init>("No valid CRL for distribution point found.", var29);
                     break;
                  }

                  var9 = true;
                  ++var10;
               }
            }

            if (var6.getCertStatus() == 11 && !var7.isAllReasons()) {
               label113: {
                  label112: {
                     label159: {
                        DERObject var38;
                        try {
                           var38 = (new ASN1InputStream(((X500Principal)attrCert.getIssuer().getPrincipals()[0]).getEncoded())).readObject();
                        } catch (Exception var19) {
                           Exception attrCert1 = var19;

                           try {
                              throw new AnnotatedException("Issuer from certificate for CRL could not be reencoded.", attrCert1);
                           } catch (AnnotatedException var15) {
                              var33 = var15;
                              var35 = false;
                              break label159;
                           }
                        }

                        DERObject var30 = var38;

                        DistributionPoint var40;
                        try {
                           var40 = new DistributionPoint;
                        } catch (AnnotatedException var18) {
                           var33 = var18;
                           var35 = false;
                           break label159;
                        }

                        DistributionPoint var31 = var40;

                        DistributionPoint var10003;
                        DistributionPointName var10004;
                        X509AttributeCertificate var39;
                        try {
                           var39 = attrCert;
                           var10002 = paramsPKIX;
                           var10003 = var31;
                           var10004 = new DistributionPointName;
                        } catch (AnnotatedException var17) {
                           var33 = var17;
                           var35 = false;
                           break label159;
                        }

                        DistributionPointName var10005 = var10004;
                        byte var10006 = 0;

                        try {
                           var10005.<init>(var10006, new GeneralNames(new GeneralName(4, var30)));
                           var10003.<init>(var10004, (ReasonFlags)null, (GeneralNames)null);
                           checkCRL(var40, var39, (ExtendedPKIXParameters)var10002.clone(), validDate, issuerCert, var6, var7, certPathCerts);
                           break label112;
                        } catch (AnnotatedException var16) {
                           var33 = var16;
                           var35 = false;
                        }
                     }

                     AnnotatedException attrCert2 = var33;
                     var8 = new AnnotatedException.<init>("No valid CRL for distribution point found.", attrCert2);
                     break label113;
                  }

                  var9 = true;
               }
            }

            if (!var9) {
               throw new ExtCertPathValidatorException("No valid CRL found.", var8);
            }

            if (var6.getCertStatus() != 11) {
               String attrCert3 = "Attribute certificate revocation after " + var6.getRevocationDate();
               attrCert3 = attrCert3 + ", reason: " + RFC3280CertPathUtilities.crlReasons[var6.getCertStatus()];
               throw new CertPathValidatorException(attrCert3);
            }

            if (!var7.isAllReasons() && var6.getCertStatus() == 11) {
               var6.setCertStatus(12);
            }

            if (var6.getCertStatus() == 12) {
               throw new CertPathValidatorException("Attribute certificate status could not be determined.");
            }
         } else if (attrCert.getExtensionValue(CRL_DISTRIBUTION_POINTS) != null || attrCert.getExtensionValue(AUTHORITY_INFO_ACCESS) != null) {
            throw new CertPathValidatorException("No rev avail extension is set, but also an AC revocation pointer.");
         }
      }

   }

   protected static void additionalChecks(X509AttributeCertificate attrCert, ExtendedPKIXParameters pkixParams) {
      Iterator var2 = pkixParams.getProhibitedACAttributes().iterator();

      while(var2.hasNext()) {
         String var3;
         if (attrCert.getAttributes(var3 = (String)var2.next()) != null) {
            throw new CertPathValidatorException("Attribute certificate contains prohibited attribute: " + var3 + ".");
         }
      }

      Iterator pkixParams1 = pkixParams.getNecessaryACAttributes().iterator();

      while(pkixParams1.hasNext()) {
         String var5;
         if (attrCert.getAttributes(var5 = (String)pkixParams1.next()) == null) {
            throw new CertPathValidatorException("Attribute certificate does not contain necessary attribute: " + var5 + ".");
         }
      }

   }

   protected static void processAttrCert5(X509AttributeCertificate attrCert, ExtendedPKIXParameters pkixParams) {
      try {
         attrCert.checkValidity(CertPathValidatorUtilities.getValidDate(pkixParams));
      } catch (CertificateExpiredException var2) {
         throw new ExtCertPathValidatorException("Attribute certificate is not valid.", var2);
      } catch (CertificateNotYetValidException var3) {
         throw new ExtCertPathValidatorException("Attribute certificate is not valid.", var3);
      }
   }

   protected static void processAttrCert4(X509Certificate acIssuerCert, ExtendedPKIXParameters pkixParams) {
      Set var10000 = pkixParams.getTrustedACIssuers();
      boolean pkixParams1 = false;
      Iterator var2 = var10000.iterator();

      while(true) {
         TrustAnchor var3;
         do {
            if (!var2.hasNext()) {
               if (pkixParams1) {
                  return;
               }

               throw new CertPathValidatorException("Attribute certificate issuer is not directly trusted.");
            }

            var3 = (TrustAnchor)var2.next();
         } while(!acIssuerCert.getSubjectX500Principal().getName("RFC2253").equals(var3.getCAName()) && !acIssuerCert.equals(var3.getTrustedCert()));

         pkixParams1 = true;
      }
   }

   protected static void processAttrCert3(X509Certificate acIssuerCert, ExtendedPKIXParameters pkixParams) {
      if (acIssuerCert.getKeyUsage() != null && !acIssuerCert.getKeyUsage()[0] && !acIssuerCert.getKeyUsage()[1]) {
         throw new CertPathValidatorException("Attribute certificate issuer public key cannot be used to validate digital signatures.");
      } else if (acIssuerCert.getBasicConstraints() != -1) {
         throw new CertPathValidatorException("Attribute certificate issuer is also a public key certificate issuer.");
      }
   }

   protected static CertPathValidatorResult processAttrCert2(CertPath certPath, ExtendedPKIXParameters pkixParams) {
      CertPathValidator var10000;
      try {
         var10000 = CertPathValidator.getInstance("PKIX", "INFOSEC");
      } catch (NoSuchProviderException var4) {
         throw new ExtCertPathValidatorException("Support class could not be created.", var4);
      } catch (NoSuchAlgorithmException var5) {
         throw new ExtCertPathValidatorException("Support class could not be created.", var5);
      }

      try {
         return var10000.validate(certPath, pkixParams);
      } catch (CertPathValidatorException var2) {
         throw new ExtCertPathValidatorException("Certification path for issuer certificate of attribute certificate could not be validated.", var2);
      } catch (InvalidAlgorithmParameterException var3) {
         throw new RuntimeException(var3.getMessage());
      }
   }

   protected static CertPath processAttrCert1(X509AttributeCertificate attrCert, ExtendedPKIXParameters pkixParams) {
      CertPathBuilderResult var2 = null;
      HashSet var3;
      var3 = new HashSet.<init>();
      IOException var10000;
      X509CertStoreSelector var10001;
      IOException attrCert1;
      AnnotatedException attrCert2;
      AnnotatedException var32;
      boolean var33;
      boolean var34;
      if (attrCert.getHolder().getIssuer() != null) {
         X509CertStoreSelector var4;
         var10001 = var4 = new X509CertStoreSelector;
         var4.<init>();
         var10001.setSerialNumber(attrCert.getHolder().getSerialNumber());
         Principal[] var5 = attrCert.getHolder().getIssuer();
         int var6 = 0;

         while(true) {
            if (var6 >= var5.length) {
               if (var3.isEmpty()) {
                  throw new CertPathValidatorException("Public key certificate specified in base certificate ID for attribute certificate cannot be found.");
               }
               break;
            }

            label123: {
               label122: {
                  label133: {
                     try {
                        var33 = var5[var6] instanceof X500Principal;
                     } catch (AnnotatedException var21) {
                        var32 = var21;
                        var34 = false;
                        break label122;
                     } catch (IOException var22) {
                        var10000 = var22;
                        var34 = false;
                        break label133;
                     }

                     if (var33) {
                        try {
                           var4.setIssuer(((X500Principal)var5[var6]).getEncoded());
                        } catch (AnnotatedException var19) {
                           var32 = var19;
                           var34 = false;
                           break label122;
                        } catch (IOException var20) {
                           var10000 = var20;
                           var34 = false;
                           break label133;
                        }
                     }

                     try {
                        var3.addAll(CertPathValidatorUtilities.findCertificates(var4, pkixParams.getStores()));
                        break label123;
                     } catch (AnnotatedException var17) {
                        var32 = var17;
                        var34 = false;
                        break label122;
                     } catch (IOException var18) {
                        var10000 = var18;
                        var34 = false;
                     }
                  }

                  attrCert1 = var10000;
                  throw new ExtCertPathValidatorException("Unable to encode X500 principal.", attrCert1);
               }

               attrCert2 = var32;
               throw new ExtCertPathValidatorException("Public key certificate for attribute certificate cannot be searched.", attrCert2);
            }

            ++var6;
         }
      }

      if (attrCert.getHolder().getEntityNames() != null) {
         X509CertStoreSelector attrCert3;
         attrCert3 = new X509CertStoreSelector.<init>();
         Principal[] var29 = attrCert.getHolder().getEntityNames();

         for(int var31 = 0; var31 < var29.length; ++var31) {
            label99: {
               label136: {
                  try {
                     var33 = var29[var31] instanceof X500Principal;
                  } catch (AnnotatedException var15) {
                     var32 = var15;
                     var34 = false;
                     break label136;
                  } catch (IOException var16) {
                     var10000 = var16;
                     var34 = false;
                     break label99;
                  }

                  if (var33) {
                     try {
                        attrCert3.setIssuer(((X500Principal)var29[var31]).getEncoded());
                     } catch (AnnotatedException var13) {
                        var32 = var13;
                        var34 = false;
                        break label136;
                     } catch (IOException var14) {
                        var10000 = var14;
                        var34 = false;
                        break label99;
                     }
                  }

                  try {
                     var3.addAll(CertPathValidatorUtilities.findCertificates(attrCert3, pkixParams.getStores()));
                     continue;
                  } catch (AnnotatedException var11) {
                     var32 = var11;
                     var34 = false;
                  } catch (IOException var12) {
                     var10000 = var12;
                     var34 = false;
                     break label99;
                  }
               }

               attrCert2 = var32;
               throw new ExtCertPathValidatorException("Public key certificate for attribute certificate cannot be searched.", attrCert2);
            }

            attrCert1 = var10000;
            throw new ExtCertPathValidatorException("Unable to encode X500 principal.", attrCert1);
         }

         if (var3.isEmpty()) {
            throw new CertPathValidatorException("Public key certificate specified in entity name for attribute certificate cannot be found.");
         }
      }

      ExtendedPKIXBuilderParameters attrCert4 = (ExtendedPKIXBuilderParameters)ExtendedPKIXBuilderParameters.getInstance(pkixParams);
      ExtCertPathValidatorException pkixParams1 = null;
      Iterator var28 = var3.iterator();

      while(var28.hasNext()) {
         var10001 = new X509CertStoreSelector();
         var10001.setCertificate((X509Certificate)var28.next());
         attrCert4.setTargetConstraints(var10001);

         CertPathBuilder var36;
         try {
            var36 = CertPathBuilder.getInstance("PKIX", "INFOSEC");
         } catch (NoSuchProviderException var7) {
            throw new ExtCertPathValidatorException("Support class could not be created.", var7);
         } catch (NoSuchAlgorithmException var8) {
            throw new ExtCertPathValidatorException("Support class could not be created.", var8);
         }

         CertPathBuilderResult var37;
         try {
            var37 = var36.build(ExtendedPKIXBuilderParameters.getInstance(attrCert4));
         } catch (CertPathBuilderException var9) {
            ExtCertPathValidatorException var30;
            var30 = new ExtCertPathValidatorException.<init>("Certification path for public key certificate of attribute certificate could not be build.", var9);
            pkixParams1 = var30;
            continue;
         } catch (InvalidAlgorithmParameterException var10) {
            throw new RuntimeException(var10.getMessage());
         }

         var2 = var37;
      }

      if (pkixParams1 == null) {
         return var2.getCertPath();
      } else {
         throw pkixParams1;
      }
   }

   private static void checkCRL(DistributionPoint dp, X509AttributeCertificate attrCert, ExtendedPKIXParameters paramsPKIX, Date validDate, X509Certificate issuerCert, CertStatus certStatus, ReasonsMask reasonMask, List certPathCerts) {
      if (attrCert.getExtensionValue(X509Extensions.NoRevAvail.getId()) == null) {
         Date issuerCert1;
         issuerCert1 = new Date.<init>(System.currentTimeMillis());
         if (validDate.getTime() > issuerCert1.getTime()) {
            throw new AnnotatedException("Validation time is in future.");
         } else {
            Set var10000 = CertPathValidatorUtilities.getCompleteCRLs(dp, attrCert, issuerCert1, paramsPKIX);
            boolean var8 = false;
            AnnotatedException var9 = null;
            Iterator var10 = var10000.iterator();

            while(var10.hasNext() && certStatus.getCertStatus() == 11 && !reasonMask.isAllReasons()) {
               AnnotatedException var28;
               label127: {
                  boolean var10001;
                  X509CRL var29;
                  try {
                     var29 = (X509CRL)var10.next();
                  } catch (AnnotatedException var26) {
                     var28 = var26;
                     var10001 = false;
                     break label127;
                  }

                  X509CRL var11 = var29;

                  ReasonsMask var30;
                  try {
                     var30 = RFC3280CertPathUtilities.processCRLD(var29, dp);
                  } catch (AnnotatedException var25) {
                     var28 = var25;
                     var10001 = false;
                     break label127;
                  }

                  ReasonsMask var12 = var30;

                  boolean var31;
                  try {
                     var31 = var30.hasNewReasons(reasonMask);
                  } catch (AnnotatedException var24) {
                     var28 = var24;
                     var10001 = false;
                     break label127;
                  }

                  if (!var31) {
                     continue;
                  }

                  PublicKey var32;
                  ExtendedPKIXParameters var33;
                  try {
                     var33 = paramsPKIX;
                     var32 = RFC3280CertPathUtilities.processCRLG(var11, RFC3280CertPathUtilities.processCRLF(var11, attrCert, (X509Certificate)null, (PublicKey)null, paramsPKIX, certPathCerts));
                  } catch (AnnotatedException var23) {
                     var28 = var23;
                     var10001 = false;
                     break label127;
                  }

                  PublicKey var13 = var32;
                  X509CRL var14 = null;

                  try {
                     var31 = var33.isUseDeltasEnabled();
                  } catch (AnnotatedException var22) {
                     var28 = var22;
                     var10001 = false;
                     break label127;
                  }

                  if (var31) {
                     try {
                        var29 = RFC3280CertPathUtilities.processCRLH(CertPathValidatorUtilities.getDeltaCRLs(issuerCert1, paramsPKIX, var11), var13);
                     } catch (AnnotatedException var21) {
                        var28 = var21;
                        var10001 = false;
                        break label127;
                     }

                     var14 = var29;
                  }

                  int var35;
                  try {
                     var35 = paramsPKIX.getValidityModel();
                  } catch (AnnotatedException var20) {
                     var28 = var20;
                     var10001 = false;
                     break label127;
                  }

                  if (var35 != 1) {
                     long var34;
                     long var36;
                     try {
                        var36 = attrCert.getNotAfter().getTime();
                        var34 = var11.getThisUpdate().getTime();
                     } catch (AnnotatedException var19) {
                        var28 = var19;
                        var10001 = false;
                        break label127;
                     }

                     if (var36 < var34) {
                        try {
                           throw new AnnotatedException("No valid CRL for current time found.");
                        } catch (AnnotatedException var15) {
                           var28 = var15;
                           var10001 = false;
                           break label127;
                        }
                     }
                  }

                  try {
                     RFC3280CertPathUtilities.processCRLB1(dp, attrCert, var11);
                     RFC3280CertPathUtilities.processCRLB2(dp, attrCert, var11);
                     RFC3280CertPathUtilities.processCRLC(var14, var11, paramsPKIX);
                     RFC3280CertPathUtilities.processCRLI(validDate, var14, attrCert, certStatus, paramsPKIX);
                     RFC3280CertPathUtilities.processCRLJ(validDate, var11, attrCert, certStatus);
                     var35 = certStatus.getCertStatus();
                  } catch (AnnotatedException var18) {
                     var28 = var18;
                     var10001 = false;
                     break label127;
                  }

                  if (var35 == 8) {
                     try {
                        certStatus.setCertStatus(11);
                     } catch (AnnotatedException var17) {
                        var28 = var17;
                        var10001 = false;
                        break label127;
                     }
                  }

                  try {
                     reasonMask.addReasons(var12);
                  } catch (AnnotatedException var16) {
                     var28 = var16;
                     var10001 = false;
                     break label127;
                  }

                  var8 = true;
                  continue;
               }

               var9 = var28;
            }

            if (!var8) {
               throw var9;
            }
         }
      }
   }
}
